
From: Milton Miller <miltonm@bga.com>

Check if a vterm was registered before accepting it as a console.
Check that a slot hasn't been probed with a tty in hvc_instantiate().
Check that a slot hasn't been free'ed when handing out console device.

Signed-off-by: Milton Miller <miltonm@bga.com>
Signed-off-by: Anton Blanchard <anton@samba.org>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 drivers/char/hvc_console.c |   18 ++++++++++++++++++
 1 files changed, 18 insertions(+)

diff -puN drivers/char/hvc_console.c~hvc_console-add-some-sanity-checks drivers/char/hvc_console.c
--- 25/drivers/char/hvc_console.c~hvc_console-add-some-sanity-checks	Wed Jul  6 14:00:35 2005
+++ 25-akpm/drivers/char/hvc_console.c	Wed Jul  6 14:00:35 2005
@@ -192,12 +192,21 @@ void hvc_console_print(struct console *c
 
 static struct tty_driver *hvc_console_device(struct console *c, int *index)
 {
+	if (vtermnos[c->index] == -1)
+		return NULL;
+
 	*index = c->index;
 	return hvc_driver;
 }
 
 static int __init hvc_console_setup(struct console *co, char *options)
 {
+	if (co->index < 0 || co->index >= MAX_NR_HVC_CONSOLES)
+		return -ENODEV;
+
+	if (vtermnos[co->index] == -1)
+		return -ENODEV;
+
 	return 0;
 }
 
@@ -227,12 +236,21 @@ console_initcall(hvc_console_init);
  */
 int hvc_instantiate(uint32_t vtermno, int index)
 {
+	struct hvc_struct *hp;
+
 	if (index < 0 || index >= MAX_NR_HVC_CONSOLES)
 		return -1;
 
 	if (vtermnos[index] != -1)
 		return -1;
 
+	/* make sure no no tty has been registerd in this index */
+	hp = hvc_get_by_index(index);
+	if (hp) {
+		kobject_put(&hp->kobj);
+		return -1;
+	}
+
 	vtermnos[index] = vtermno;
 
 	/* reserve all indices upto and including this index */
_
